/*
    $Id: apply_to_all.h 92 2010-02-27 19:14:17Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _APPLY_TO_ALL_H
#define _APPLY_TO_ALL_H

#include <cmath>

#include "lambert.h"

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>

namespace {

using namespace boost::numeric::ublas;

};

/* examples from effective ublas are used here */
template<class OP, class E> BOOST_UBLAS_INLINE typename vector_unary_traits<E, OP>::result_type apply_to_all (const vector_expression<E> &e, const OP& op = OP() ) {
	typedef typename vector_unary_traits<E, OP>::expression_type expression_type;
	return expression_type (e ());
}
template<class OP, class E> BOOST_UBLAS_INLINE typename matrix_unary1_traits<E, OP>::result_type apply_to_all (const matrix_expression<E> &e, const OP& op = OP() ) {
	typedef typename matrix_unary1_traits<E, OP>::expression_type expression_type;
	return expression_type (e ());
}

namespace functor {
	template <class T> class inv {
	public:
		typedef T value_type;
		typedef T result_type;
		inv() { }
		static result_type apply( const value_type& x) {
			return ( x != value_type(0) ? value_type(1) / x : x );
		}
	};
	template <class T> class exp {
	public:
		typedef T value_type;
		typedef T result_type;
		exp() { }
		static result_type apply( const value_type& x) {
			return std::exp( x );
		}
	};
	template <class T> class log {
	public:
		typedef T value_type;
		typedef T result_type;
		log() { }
		static result_type apply( const value_type& x) {
			return std::log( x );
		}
	};
	template <class T> class lambertW {
	public:
		typedef T value_type;
		typedef T result_type;
		lambertW() { }
		static result_type apply( const value_type& x) {
			return lambert_W0(x);
		}
	};
	template <class T> class sqrt {
	public:
		typedef T value_type;
		typedef T result_type;
		sqrt() { }
		static result_type apply( const value_type& x) {
			return std::sqrt(x);
		}
	};
	template <class T> class m1mx {
	public:
		typedef T value_type;
		typedef T result_type;
		m1mx() { }
		static result_type apply( const value_type& x) {
			return value_type(1) - x;
		}
	};
	template <class T> class m1px {
	public:
		typedef T value_type;
		typedef T result_type;
		m1px() { }
		static result_type apply( const value_type& x) {
			return value_type(1) + x;
		}
	};
}

#endif // _APPLY_TO_ALL_H
