/*
    $Id: lambert.h 91 2010-02-26 15:35:11Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _LAMBERT_H
#define _LAMBERT_H

#include <cmath>
#include <boost/math/special_functions.hpp>

template<class T, class Policy> struct lambert_W0_series {
	// we must define a result_type typedef:
	typedef T result_type;
public:
	lambert_W0_series( T z ): z_(z), last_(0.0) {
	}

	result_type operator()() {
		result_type ew   = std::exp( last_ );
		result_type diff = -(last_*ew - z_) / ( (ew*(last_+result_type(1))) - ((last_+result_type(2))*(last_*ew-z_))/(2*last_+result_type(2)) );
		last_ += diff;
		if( std::abs( diff / last_ ) < std::numeric_limits< result_type >::epsilon() ) // rounding error
			return result_type(0);
		return diff;
	}

private:
	T last_;
	const T z_;
};

template <class T, class Policy> T lambert_W0(T z, const Policy& p) {
	boost::uintmax_t max_iter = boost::math::policies::get_max_series_iterations<Policy>();
	lambert_W0_series<T, Policy> W0(z);
	return boost::math::tools::sum_series(W0, boost::math::policies::digits<T, Policy>(), max_iter);
}
template <class T> T lambert_W0(T z) {
	return lambert_W0( z, boost::math::policies::policy<>() );
}

#endif // _LAMBERT_H

