/*
    $Id: photometry.h 80 2010-02-21 16:20:54Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PHOTOMETRY_H
#define _PHOTOMETRY_H

#include <boost/numeric/ublas/vector.hpp>

#include "typesdef.h"

namespace photometry {
	using namespace boost::numeric::ublas;

	/**
		$$ \xi = -\frac{1}{\tau} W(-\tau\xi')$$
	*/
	template<class V1,class V2>
	vector< typename V1::value_type >
	flux_nonlinearity_correction( const vector_expression<V1>& flux, const vector_expression<V2>& deadtime ) {
		typedef typename V1::value_type value_type;
		
		assert( flux().size() == deadtime().size() );

		return -element_div( apply_to_all( -element_prod( flux, deadtime ), functor::lambertW< value_type >() ), deadtime );
	}

	/**
		$$ \xi = (1 - s), \xi'$$
	*/
	template<class V1,class V2>
	vector< typename V1::value_type >
	flux_scatter_correction( const vector_expression<V1>& flux, const vector_expression<V2>& scatter ) {
		typedef typename V1::value_type value_type;

		assert( flux().size() == scatter().size() );

		return element_prod(
			apply_to_all(
				scatter,
				functor::m1mx<value_type>()
				),
			flux
			);
	}

	template<class V1,class V2,class V3>
	vector< typename V1::value_type >
	variance_nonpoisson_correction (
		const vector_expression<V1>& flux,
		const vector_expression<V2>& deadtime,
		const vector_expression<V3>& nonpoiss ) {
		typedef typename V1::value_type value_type;

		assert( flux().size() == deadtime().size() );
		assert( flux().size() == nonpoiss().size() );

		vector< typename V1::value_type > input_flux = element_prod( flux, apply_to_all( -element_prod(deadtime,flux),functor::exp<value_type>() ) );
		
		return element_prod(
			element_prod(
				input_flux,
				apply_to_all(
					element_prod( input_flux, deadtime )*( -2.0 ),
					functor::m1px<value_type>()
					)
				),
			nonpoiss
			);
	}

	template<class V1,class V2>
	vector< typename V1::value_type >
	variance_nonlinearity_correction ( const vector_expression<V1>& flux, const vector_expression<V2>& deadtime ) {
		typedef typename V1::value_type value_type;

		assert( flux().size() == deadtime().size() );

		return element_prod( apply_to_all( -element_prod( deadtime, flux ), functor::exp<value_type>() ), apply_to_all( element_prod( deadtime, flux ), functor::m1mx<value_type>() ) );
	}
};

#endif // _PHOTOMETRY_H
