/*
    $Id: spectral_factory.h 101 2010-03-02 08:36:15Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _SPECTRAL_FACTORY_H
#define _SPECTRAL_FACTORY_H

#include <map>
#include <string>

#include <boost/filesystem/path.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include "nocase_less.h"
#include "typesdef.h"
#include "vector_generator.h"
#include "weif.h"

using namespace boost::numeric;

class spectral_factory{
public:
	typedef Weights weight_type;
	typedef std::map< std::string, weight_type, nocase_less< std::string > > storage_type;

	typedef storage_type::const_iterator const_iterator;
private:
	boost::filesystem::path m_mass_response;
	boost::filesystem::path m_ccd_response;
	boost::filesystem::path m_wdir;

	vector_generator< lg_generator > m_basis_grid;

	storage_type m_storage;
private:
	const boost::filesystem::path get_spectrum_filename( const std::string& spectype ) const;
public:
	spectral_factory(const boost::filesystem::path& wd, const boost::filesystem::path& mass_response, const boost::filesystem::path& ccd_response );
	~spectral_factory();

	const weight_type& spectral( const std::string& spectype, const ublas::vector<double>& inner_diam, const ublas::vector<double>& outer_diam, double dimmbase, double dimmsize );

	const const_iterator begin() const { return m_storage.begin(); }
	const const_iterator end()   const { return m_storage.end(); }
};

#endif //_SPECTRAL_FACTORY_H
