#ifndef _TABLE_STORAGE_H
#define _TABLE_STORAGE_H

#include <vector>

#include <boost/iterator/transform_iterator.hpp>

template< class T > class table_storage {
public:
	typedef T value_type;
	typedef std::vector<T> container_type;
	typedef typename container_type::size_type size_type;
	typedef typename container_type::const_reference const_reference;

	typedef typename container_type::iterator       iterator;
	typedef typename container_type::const_iterator const_iterator;

private:
	container_type m_storage;
public:
	table_storage() {
	}
	table_storage( size_type reserve ) {
		m_storage.reserve( reserve );
	}
	void swap( table_storage& ts ) throw() {
		m_storage.swap( ts.m_storage );
	}
	table_storage( const table_storage& ts ): m_storage( ts.m_storage ) {
	}
	table_storage& operator=( const table_storage& ts ) {
		 table_storage temp( ts );
		 temp.swap( *this );
		 return *this;
	}
	~table_storage() {
	}

	const_reference back()  const { return m_storage.back(); }
	const_reference front() const { return m_storage.front(); }
	void push_back( const T& x ) { m_storage.push_back( x ); }
	void pop_back() { m_storage.pop_back(); }

	iterator begin() { return m_storage.begin(); }
	iterator end()   { return m_storage.end(); }
	const_iterator begin() const { return m_storage.begin(); }
	const_iterator end()   const { return m_storage.end(); }

	size_type size() const { return m_storage.size(); }
	void clear() { return m_storage.clear(); }

	template<class Predicate> void unstable_remove_if( Predicate pred ) {
		typename container_type::iterator it = m_storage.begin();
		
		while( it != m_storage.end() ) {
			if( pred( *it ) ) {
				swap( *it, m_storage.back() );
				m_storage.pop_back();
			} else
				++it;
		}
	}
};

template<class T, class F> class table_storage_column {
public:
	typedef T                subvalue_type;
	typedef table_storage<T> table_storage_type;
	typedef F                proxy_function_type;

	typedef typename table_storage_type::const_iterator const_subiterator;

	typedef boost::transform_iterator< proxy_function_type, const_subiterator > const_iterator;
private:
	const table_storage<T>& m_storage;
	proxy_function_type m_proxy_function;
public:
	table_storage_column( const table_storage<T>& storage, const F& proxy_function ):
		m_storage( storage ), m_proxy_function( proxy_function ) {
	}

	const_iterator begin() const { return const_iterator( m_storage.begin(), m_proxy_function ); }
	const_iterator end()   const { return const_iterator( m_storage.end(), m_proxy_function ); }
};

template<class T> void swap( table_storage<T>& t, table_storage<T>& u ) {
	t.swap( u );
}

#endif // _TABLE_STORAGE_H
