/*
    $Id: typesdef.h 95 2010-02-28 16:07:27Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _TYPESDEF_H
#define _TYPESDEF_H
#include <vector>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/matrix_proxy.hpp>
#include <boost/numeric/ublas/vector_proxy.hpp>

#include <yalpa/spheric.h>

#include "apply_to_all.h"

struct object {
	unsigned int hd;
	double       vmag;
	double       bvind;
	std::string  name;
	yalpa::equatorial<double> coordinates;
	std::string  sclass;
};

class datum {
public:
    double  mean;
    double  errs;
    datum( double m = 0.0, double e = 0.0 ) : mean(m), errs(e) {}
};

using namespace boost::numeric::ublas;

template<class ME> matrix_vector_slice< const ME > diag_elements( const ME& x ){
    typename ME::size_type r = std::min( x.size1(), x.size2() );
    slice s(0, 1, r);
    return matrix_vector_slice< const ME >( x, s, s );
}

struct is_positive_number {
	bool operator()( const double& x ) {
		return ! ( std::isnan( x ) || std::isinf( x ) || ( x < 0 ) );
	}
};

#endif // _TYPESDEF_H
