/*
    $Id: sun.cpp 13 2010-01-28 15:27:59Z matwey $
    Copyright (C) 2010  Matwey V. Kornilov <matwey.kornilov@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <yalpa/sidtime.h>
#include <yalpa/sun.h>

#include <yalpa/io.h>

#include <cmath>

double yalpa::sun_longitude( const std::time_t& epoch ) {
	return sun_longitude( julian( epoch ) );
}

double yalpa::sun_longitude( double julian ) {
	/* from Jean Meeus */
	double t = (julian - 2451545.0) / 36525.0; // Julian centures
	double meanLongitude = M_PI / 180.0 * ( 280.46645 + 36000.76983 * t + 0.0003032 * t*t );
	double meanAnomaly   = M_PI / 180.0 * ( 357.52910 + 35999.05030 * t - 0.0001559 * t*t - 0.00000048 * t*t*t );
	double center        = M_PI / 180.0 * ( (1.914600 - 0.004817 * t - 0.000014 * t*t) * sin(meanAnomaly) + (0.019933 - 0.000101 * t) * sin(2*meanAnomaly) + 0.000290 * sin(3*meanAnomaly) );
	/* it is geometrical longitude. we don't corrent for nutation and the aberration */
	double longitude = std::fmod( meanLongitude + center, 2*M_PI );
	return ( longitude < 0 ? longitude + 2*M_PI : longitude );
}
yalpa::ecliptic<double> yalpa::sun_position( double julian ) {
	return ecliptic<double>( sun_longitude(julian), 0.0 );
}
yalpa::ecliptic<double> yalpa::sun_position( const std::time_t& epoch ) {
	return sun_position(julian(epoch));
}
double yalpa::sun_longitude( const boost::posix_time::ptime& time ) {
	return sun_longitude( julian( time ) );
}
yalpa::ecliptic<double> yalpa::sun_position( const boost::posix_time::ptime& time ) {
	return sun_position(julian(time));
}
